/* Formbar Style */

/* Make all field elements expand to 100% on default */
input, select, textarea {
  width: 100%;
  /* Fix width and height of the fields by setting it to the border-box modell
   * (CSS3) Does not work in IE */
  height: 32px !important;
  box-sizing: border-box;
  -webkit-box-sizing:border-box;
  -moz-box-sizing: border-box;
}

select[multiple], textarea {
  /* Reset heigt of the field elements which have been modified by the
   * border-box modell */
  height: auto !important;
}

div.readonlyfield {
  border-radius: 3px 3px 3px 3px;
  background-color: #e0e0e0;
  height: 32px;
  margin-bottom: 10px;
}

/* Helptexts */
.text-help {
  position: relative;
  top: -5px;
  font-size: 0.8em;
  color: #707070;
}

.text-error {
  position: relative;
  top: -5px;
  font-size: 0.8em;
}

/* Button */
button {
  margin-right: 15px;
}

.button-pane {
  margin-top: 20px;
  background-color: #e0e0e0;
}
